@@ -26,6 +26,7 @@
 
 #include <string.h>
 #include <stdlib.h>
+#include <unistd.h>
 
 #include "gettext.h"
 #include <locale.h>
@@ -78,64 +79,63 @@
 struct directory_entry {
 	const char *lang_dir;
 	const char *source_encoding;
-	const char *standard_output_encoding;
 };
 
 static struct directory_entry directory_table[] = {
-	{ "C",		"ISO-8859-1",	"ANSI_X3.4-1968"	}, /* English */
-	{ "POSIX",	"ISO-8859-1",	"ANSI_X3.4-1968"	}, /* English */
-	{ "da",		"ISO-8859-1",	"ISO-8859-1"		}, /* Danish */
-	{ "de",		"ISO-8859-1",	"ISO-8859-1"		}, /* German */
-	{ "en",		"ISO-8859-1",	"ISO-8859-1"		}, /* English */
-	{ "es",		"ISO-8859-1",	"ISO-8859-1"		}, /* Spanish */
-	{ "et",		"ISO-8859-1",	"ISO-8859-1"		}, /* Estonian */
-	{ "fi",		"ISO-8859-1",	"ISO-8859-1"		}, /* Finnish */
-	{ "fr",		"ISO-8859-1",	"ISO-8859-1"		}, /* French */
-	{ "ga",		"ISO-8859-1",	"ISO-8859-1"		}, /* Irish */
-	{ "gl",		"ISO-8859-1",	"ISO-8859-1"		}, /* Galician */
-	{ "id",		"ISO-8859-1",	"ISO-8859-1"		}, /* Indonesian */
-	{ "is",		"ISO-8859-1",	"ISO-8859-1"		}, /* Icelandic */
-	{ "it",		"ISO-8859-1",	"ISO-8859-1"		}, /* Italian */
-	{ "nb",		"ISO-8859-1",	"ISO-8859-1"		}, /* Norwegian Bokmål */
-	{ "nl",		"ISO-8859-1",	"ISO-8859-1"		}, /* Dutch */
-	{ "nn",		"ISO-8859-1",	"ISO-8859-1"		}, /* Norwegian Nynorsk */
-	{ "no",		"ISO-8859-1",	"ISO-8859-1"		}, /* Norwegian */
-	{ "pt",		"ISO-8859-1",	"ISO-8859-1"		}, /* Portuguese */
-	{ "sv",		"ISO-8859-1",	"ISO-8859-1"		}, /* Swedish */
+	{ "C",		"ISO-8859-1"	}, /* English */
+	{ "POSIX",	"ISO-8859-1"	}, /* English */
+	{ "da",		"ISO-8859-1"	}, /* Danish */
+	{ "de",		"ISO-8859-1"	}, /* German */
+	{ "en",		"ISO-8859-1"	}, /* English */
+	{ "es",		"ISO-8859-1"	}, /* Spanish */
+	{ "et",		"ISO-8859-1"	}, /* Estonian */
+	{ "fi",		"ISO-8859-1"	}, /* Finnish */
+	{ "fr",		"ISO-8859-1"	}, /* French */
+	{ "ga",		"ISO-8859-1"	}, /* Irish */
+	{ "gl",		"ISO-8859-1"	}, /* Galician */
+	{ "id",		"ISO-8859-1"	}, /* Indonesian */
+	{ "is",		"ISO-8859-1"	}, /* Icelandic */
+	{ "it",		"ISO-8859-1"	}, /* Italian */
+	{ "nb",		"ISO-8859-1"	}, /* Norwegian Bokmål */
+	{ "nl",		"ISO-8859-1"	}, /* Dutch */
+	{ "nn",		"ISO-8859-1"	}, /* Norwegian Nynorsk */
+	{ "no",		"ISO-8859-1"	}, /* Norwegian */
+	{ "pt",		"ISO-8859-1"	}, /* Portuguese */
+	{ "sv",		"ISO-8859-1"	}, /* Swedish */
 
 #ifdef MULTIBYTE_GROFF
 	/* These languages require a patched version of groff with the
 	 * ascii8 and nippon devices.
 	 */
-	{ "be",		"CP1251",	"CP1251"		}, /* Belarusian */
-	{ "bg",		"CP1251",	"CP1251"		}, /* Bulgarian */
-	{ "cs",		"ISO-8859-2",	"ISO-8859-2"		}, /* Czech */
-	{ "el",		"ISO-8859-7",	"ISO-8859-7"		}, /* Greek */
-	{ "hr",		"ISO-8859-2",	"ISO-8859-2"		}, /* Croatian */
-	{ "hu",		"ISO-8859-2",	"ISO-8859-2"		}, /* Hungarian */
-	{ "ja",		"EUC-JP",	"EUC-JP"		}, /* Japanese */
-	{ "ko",		"EUC-KR",	"EUC-KR"		}, /* Korean */
-	{ "lt",		"ISO-8859-13",	"ISO-8859-13"		}, /* Lithuanian */
-	{ "lv",		"ISO-8859-13",	"ISO-8859-13"		}, /* Latvian */
-	{ "mk",		"ISO-8859-5",	"ISO-8859-5"		}, /* Macedonian */
-	{ "pl",		"ISO-8859-2",	"ISO-8859-2"		}, /* Polish */
-	{ "ro",		"ISO-8859-2",	"ISO-8859-2"		}, /* Romanian */
-	{ "ru",		"KOI8-R",	"KOI8-R"		}, /* Russian */
-	{ "sk",		"ISO-8859-2",	"ISO-8859-2"		}, /* Slovak */
-	{ "sl",		"ISO-8859-2",	"ISO-8859-2"		}, /* Slovenian */
+	{ "be",		"CP1251"	}, /* Belarusian */
+	{ "bg",		"CP1251"	}, /* Bulgarian */
+	{ "cs",		"ISO-8859-2"	}, /* Czech */
+	{ "el",		"ISO-8859-7"	}, /* Greek */
+	{ "hr",		"ISO-8859-2"	}, /* Croatian */
+	{ "hu",		"ISO-8859-2"	}, /* Hungarian */
+	{ "ja",		"EUC-JP"	}, /* Japanese */
+	{ "ko",		"EUC-KR"	}, /* Korean */
+	{ "lt",		"ISO-8859-13"	}, /* Lithuanian */
+	{ "lv",		"ISO-8859-13"	}, /* Latvian */
+	{ "mk",		"ISO-8859-5"	}, /* Macedonian */
+	{ "pl",		"ISO-8859-2"	}, /* Polish */
+	{ "ro",		"ISO-8859-2"	}, /* Romanian */
+	{ "ru",		"KOI8-R"	}, /* Russian */
+	{ "sk",		"ISO-8859-2"	}, /* Slovak */
+	{ "sl",		"ISO-8859-2"	}, /* Slovenian */
 	/* sr@latin must precede sr, due to top-down left-substring matching later */
-	{ "sr@latin",   "ISO-8859-2",   "ISO-8859-2"            }, /* Serbian Latin */
-	{ "sr",		"ISO-8859-5",	"ISO-8859-5"		}, /* Serbian */
-	{ "tr",		"ISO-8859-9",	"ISO-8859-9"		}, /* Turkish */
-	{ "uk",		"KOI8-U",	"KOI8-U"		}, /* Ukrainian */
-	{ "vi",		"TCVN5712-1",	"TCVN5712-1"		}, /* Vietnamese */
-	{ "zh_CN",	"GBK",		"GBK"			}, /* Simplified Chinese */
-	{ "zh_SG",	"GBK",		"GBK"			}, /* Simplified Chinese, Singapore */
-	{ "zh_HK",	"BIG5HKSCS",	"BIG5HKSCS"		}, /* Traditional Chinese, Hong Kong */
-	{ "zh_TW",	"BIG5",		"BIG5"			}, /* Traditional Chinese */
+	{ "sr@latin",	"ISO-8859-2"	}, /* Serbian Latin */
+	{ "sr",		"ISO-8859-5"	}, /* Serbian */
+	{ "tr",		"ISO-8859-9"	}, /* Turkish */
+	{ "uk",		"KOI8-U"	}, /* Ukrainian */
+	{ "vi",		"TCVN5712-1"	}, /* Vietnamese */
+	{ "zh_CN",	"GBK"		}, /* Simplified Chinese */
+	{ "zh_SG",	"GBK"		}, /* Simplified Chinese, Singapore */
+	{ "zh_HK",	"BIG5HKSCS"	}, /* Traditional Chinese, Hong Kong */
+	{ "zh_TW",	"BIG5"		}, /* Traditional Chinese */
 #endif /* MULTIBYTE_GROFF */
 
-	{ NULL,		NULL,		"NULL"			}
+	{ NULL,		NULL		}
 };
 
 static const char *fallback_source_encoding = "ISO-8859-1";
@@ -514,45 +514,6 @@ const char *get_source_encoding (const char *lang)
 	return fallback_source_encoding;
 }
 
-/* Return the standard output encoding for the source man page, based on the
- * directory in which it was found. This should only be used to determine
- * whether a cat page can be saved.
- *
- * The caller should free the returned string when it is finished with it.
- */
-char *get_standard_output_encoding (const char *lang)
-{
-	const struct directory_entry *entry;
-	const char *dot;
-
-	if (!lang || !*lang) {
-		/* Guess based on the locale. */
-		lang = setlocale (LC_MESSAGES, NULL);
-		if (!lang)
-			return NULL;
-	}
-
-	dot = strchr (lang, '.');
-	if (dot) {
-		/* The cat directory will have a corresponding name to the
-		 * man directory including an explicit character set, so the
-		 * pages it contains should have that encoding.
-		 */
-		char *dir_encoding =
-			xstrndup (dot + 1, strcspn (dot + 1, ",@"));
-		char *canonical_dir_encoding =
-			xstrdup (get_canonical_charset_name (dir_encoding));
-		free (dir_encoding);
-		return canonical_dir_encoding;
-	}
-
-	for (entry = directory_table; entry->lang_dir; ++entry)
-		if (STRNEQ (entry->lang_dir, lang, strlen (entry->lang_dir)))
-			return xstrdup (entry->standard_output_encoding);
-
-	return NULL;
-}
-
 const char *get_canonical_charset_name (const char *charset)
 {
 	const struct charset_alias_entry *entry;
@@ -604,6 +565,61 @@ const char *get_locale_charset (void)
 		return NULL;
 }
 
+/* Find a locale with this character set. This is a non-portable operation,
+ * but required to make col(1) work correctly with -E. If no locale can be
+ * found, or if none needs to be set, return NULL.
+ *
+ * The caller should free the returned string when it is finished with it.
+ */
+char *find_charset_locale (const char *charset)
+{
+	const char *canonical_charset = get_canonical_charset_name (charset);
+	char *saved_locale;
+	const char *supported_path = "/usr/share/i18n/SUPPORTED";
+	FILE *supported;
+	char *line = NULL;
+	size_t n = 0;
+	char *locale = NULL;
+
+	if (STREQ (charset, get_locale_charset ()))
+		return NULL;
+
+	supported = fopen (supported_path, "r");
+	if (!supported)
+		return NULL;
+
+	saved_locale = xstrdup (setlocale (LC_CTYPE, NULL));
+
+	while (getline (&line, &n, supported) >= 0) {
+		const char *space = strchr (line, ' ');
+		if (space) {
+			char *encoding = xstrdup (space + 1);
+			char *newline = strchr (encoding, '\n');
+			if (newline)
+				*newline = 0;
+			if (STREQ (canonical_charset,
+				   get_canonical_charset_name (encoding))) {
+				locale = xstrndup (line, space - line);
+				/* Is this locale actually installed? */
+				if (setlocale (LC_CTYPE, locale)) {
+					free (encoding);
+					free (line);
+					goto out;
+				} else
+					locale = NULL;
+			}
+			free (encoding);
+		}
+		free (line);
+		line = NULL;
+	}
+
+out:
+	setlocale (LC_CTYPE, saved_locale);
+	fclose (supported);
+	return locale;
+}
+
 /* Can we take this input encoding and produce this output encoding, perhaps
  * with the help of some iconv pipes? */
 static int compatible_encodings (const char *input, const char *output)
@@ -658,8 +674,17 @@ const char *get_default_device (const char *locale_charset,
 {
 	const struct charset_entry *entry;
 
-	if (get_groff_preconv ())
-		return "utf8";
+	if (get_groff_preconv ()) {
+		/* ASCII is a special case, and the only way we can get
+		 * things like bullet marks to come out right is by using
+		 * the ascii device. People using such a basic locale
+		 * probably don't want anything fancy anyway.
+		 */
+		if (locale_charset && STREQ (locale_charset, "ANSI_X3.4-1968"))
+			return "ascii";
+		else
+			return "utf8";
+	}
 
 	if (!locale_charset)
 		return fallback_default_device;
