@@ -96,7 +96,8 @@ static error_t parse_opt (int key, char *arg, struct argp_state *state)
 			return 0;
 		case 'h':
 			argp_state_help (state, state->out_stream,
-					 ARGP_HELP_STD_HELP);
+					 ARGP_HELP_STD_HELP &
+					 ~ARGP_HELP_PRE_DOC);
 			break;
 		case ARGP_KEY_ARGS:
 			files = state->argv + state->next;
@@ -133,6 +134,8 @@ int main (int argc, char **argv)
 
 	program_name = base_name (argv[0]);
 
+	init_debug ();
+
 	if (!setlocale (LC_ALL, "") && !getenv ("MAN_NO_LOCALE_WARNING"))
 		/* Obviously can't translate this. */
 		error (0, 0, "can't set the locale; make sure $LC_* and $LANG "
